package com.tzld.longarticle.recommend.server.service.filter.strategy;

import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.repository.entity.crawler.Article;
import com.tzld.longarticle.recommend.server.remote.ArticleListRemoteService;
import com.tzld.longarticle.recommend.server.service.AccountContentPoolConfigService;
import com.tzld.longarticle.recommend.server.service.filter.FilterParam;
import com.tzld.longarticle.recommend.server.service.filter.FilterResult;
import com.tzld.longarticle.recommend.server.service.filter.FilterStrategy;
import com.tzld.longarticle.recommend.server.util.TitleSimilarCheckUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

/**
 * @author dyp
 */
@Component
@Slf4j
public class HistoryTitleStrategy implements FilterStrategy {

    @Autowired
    private ArticleListRemoteService articleListRemoteService;
    @Autowired
    private AccountContentPoolConfigService accountContentPoolConfigService;

    private static final List<Integer> firstSecondIndex = Arrays.asList(1, 2);
    private static final List<Integer> allIndex = Arrays.asList(1, 2, 3, 4, 5, 6, 7, 8);


    @Override
    public FilterResult filter(FilterParam param) {
        FilterResult filterResult = new FilterResult();
        List<String> result = new ArrayList<>();
        List<Content> filterContents = new ArrayList<>();
        List<Article> firstSecondArticleList = articleListRemoteService.articleList(param.getAccountName(), firstSecondIndex);
        List<String> firstSecondTitleList = firstSecondArticleList.stream().map(Article::getTitle).distinct().collect(Collectors.toList());
        List<Article> allArticleList = articleListRemoteService.articleList(param.getAccountName(), allIndex);
        List<String> allTitleList = allArticleList.stream().map(Article::getTitle).distinct().collect(Collectors.toList());

        List<String> firstSecondContentPool = new ArrayList<>();
        String[] contentPoolConfig = accountContentPoolConfigService.getContentPools(param.getAccountName());
        if (Objects.nonNull(contentPoolConfig)) {
            firstSecondContentPool = Arrays.asList(contentPoolConfig[0], contentPoolConfig[1]);
        }
        for (Content content : param.getContents()) {
            boolean isDuplicate;
            if (CollectionUtils.isNotEmpty(firstSecondContentPool) && firstSecondContentPool.contains(content.getContentPoolType())) {
                // 四个内容池 配置 判断头条，次头条
                isDuplicate = TitleSimilarCheckUtil.isDuplicateContent(content.getTitle(), firstSecondTitleList);
            } else {
                isDuplicate = TitleSimilarCheckUtil.isDuplicateContent(content.getTitle(), allTitleList);
            }
            if (!isDuplicate) {
                result.add(content.getId());
            } else {
                content.setFilterReason("历史已发布文章");
                filterContents.add(content);
            }
        }
        filterResult.setContentIds(result);
        filterResult.setFilterContent(filterContents);
        return filterResult;
    }

}
