package com.tzld.longarticle.recommend.server.service.filter.strategy;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.longarticle.recommend.server.common.ThreadPoolFactory;
import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.remote.ArticleSensitiveRemoteService;
import com.tzld.longarticle.recommend.server.service.filter.FilterParam;
import com.tzld.longarticle.recommend.server.service.filter.FilterResult;
import com.tzld.longarticle.recommend.server.service.filter.FilterStrategy;
import com.tzld.longarticle.recommend.server.util.TitleSimilarCheckUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.List;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

@Component
@Slf4j
public class SensitiveStrategy implements FilterStrategy {

    @Autowired
    private ArticleSensitiveRemoteService articleSensitiveRemoteService;

    @ApolloJsonValue("${UnSafeTitles:[]}")
    private static List<String> UnSafeTitles;

    private final ExecutorService pool = ThreadPoolFactory.defaultPool();

    @Override
    public FilterResult filter(FilterParam param) {
        FilterResult filterResult = new FilterResult();
        List<String> result = new ArrayList<>();
        List<Content> filterContents = new ArrayList<>();

        CountDownLatch cdl = new CountDownLatch(param.getContents().size());
        List<Future<Content>> futures = new ArrayList<>();

        for (Content content : param.getContents()) {
            Future<Content> future = pool.submit(() -> {
                try {
                    boolean isSensitive = articleSensitiveRemoteService.articleSensitive(content.getTitle());
                    if (isSensitive) {
                        content.setFilterReason("安全违规");
                    } else if (TitleSimilarCheckUtil.isDuplicateContent(content.getTitle(), UnSafeTitles)) {
                        content.setFilterReason("安全违规");
                    }
                    return content;
                } finally {
                    cdl.countDown();
                }
            });
            futures.add(future);
        }
        try {
            cdl.await(5000, TimeUnit.MILLISECONDS);
        } catch (InterruptedException e) {
            log.error("filter error", e);
            return null;
        }

        for (Future<Content> f : futures) {
            try {
                Content content = f.get();
                if (StringUtils.hasText(content.getFilterReason())) {
                    filterContents.add(content);
                } else {
                    result.add(content.getId());
                }
            } catch (Exception e) {
                log.error("future get error ", e);
            }
        }
        filterResult.setContentIds(result);
        filterResult.setFilterContent(filterContents);
        return filterResult;
    }

}
