package com.tzld.longarticle.recommend.server.service.score.strategy;

import cn.hutool.core.io.resource.ResourceUtil;
import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.model.Content;
import com.tzld.longarticle.recommend.server.repository.adplatform.ChangwenArticleDatastatRepository;
import com.tzld.longarticle.recommend.server.repository.adplatform.ChangwenArticleRepository;
import com.tzld.longarticle.recommend.server.repository.entity.adplatform.ChangwenArticle;
import com.tzld.longarticle.recommend.server.repository.entity.adplatform.ChangwenArticleDatastat;
import com.tzld.longarticle.recommend.server.service.score.*;
import com.tzld.longarticle.recommend.server.util.CommonCollectionUtils;
import com.tzld.longarticle.recommend.server.util.NormalizationUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.PostConstruct;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author dyp
 */
@Component
public class ViewMultiplierStrategy implements ScoreStrategy {

    @Autowired
    ChangwenArticleRepository changwenArticleRepository;
    @Autowired
    ChangwenArticleDatastatRepository changwenArticleDatastatRepository;

    private JSONObject jsonObject;

    @PostConstruct
    public void init() {
        String cardJSON = ResourceUtil.readUtf8Str("file/AccountInfo.json");
        jsonObject = JSONObject.parseObject(cardJSON);
    }

    @Override
    public List<Score> score(ScoreParam param) {

        if (CollectionUtils.isEmpty(param.getContents())) {
            return Collections.emptyList();
        }
        List<String> channelContentIds = param.getContents().stream().map(Content::getCrawlerChannelContentId).collect(Collectors.toList());
        Map<String, ChangwenArticle> changwenArticleDTOMap = getContentIndex(channelContentIds);
        Map<String, ChangwenArticleDatastat> changwenArticleDatastatDTOMap = getContentViewCount(channelContentIds);

        List<Score> scores = CommonCollectionUtils.toList(param.getContents(), c -> {
            Score score = new Score();
            score.setContentId(c.getId());
            ChangwenArticle article = changwenArticleDTOMap.get(c.getCrawlerChannelContentId());
            ChangwenArticleDatastat datastat = changwenArticleDatastatDTOMap.get(c.getCrawlerChannelContentId());

            if (Objects.nonNull(article) && Objects.nonNull(datastat)) {
                double avgReadCount = getAvgReadCount(article.getAccountId(), article.getItemIndex());
                Integer readCount = datastat.getReadCount();
                score.setScore(NormalizationUtils.min(Math.max(readCount, 0) / avgReadCount - 1));
            } else {
                score.setScore(-1.0);
            }
            score.setStrategy(this);
            return score;
        });
        return scores;
    }

    private Map<String, ChangwenArticle> getContentIndex(List<String> channelContentIds) {
        if (CollectionUtils.isEmpty(channelContentIds)) {
            return new HashMap<>();
        }
        List<ChangwenArticle> result = changwenArticleRepository.findAllById(channelContentIds);
        if (CollectionUtils.isEmpty(result)) {
            return new HashMap<>();
        }
        return result.stream().collect(Collectors.toMap(ChangwenArticle::getId, o -> o));
    }

    private Map<String, ChangwenArticleDatastat> getContentViewCount(List<String> channelContentIds) {
        if (CollectionUtils.isEmpty(channelContentIds)) {
            return new HashMap<>();
        }
        List<ChangwenArticleDatastat> result = changwenArticleDatastatRepository.findAllById(channelContentIds);
        if (CollectionUtils.isEmpty(result)) {
            return new HashMap<>();
        }
        return result.stream().collect(Collectors.toMap(ChangwenArticleDatastat::getArticleId, o -> o));
    }

    private double getAvgReadCount(String ghId, Integer index) {
        AvgReadDTO dto = jsonObject.getObject(ghId + "_" + index, AvgReadDTO.class);
        return dto == null ? 1.0 : dto.getReadAvg();
    }
}
